@@ -230,7 +230,8 @@ type WorkflowCallEvent struct {
 	// Inputs is a map from input name to input configuration.
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#onworkflow_callinputs
 	Inputs map[*String]*WorkflowCallEventInput
-	// Secrets is a map from name of secret to secret configuration.
+	// Secrets is a map from name of secret to secret configuration. When 'secrets' is omitted, nil is set to this
+	// field.
 	// https://docs.github.com/en/actions/learn-github-actions/workflow-syntax-for-github-actions#onworkflow_callsecrets
 	Secrets map[*String]*WorkflowCallEventSecret
 	// Outputs is a map from name of output to output configuration.
@@ -734,6 +735,9 @@ type WorkflowCall struct {
 	Inputs map[string]*WorkflowCallInput
 	// Secrets is a map from secret name to secret value at 'secrets:'.
 	Secrets map[string]*WorkflowCallSecret
+	// InheritSecrets is true when 'secrets: inherit' is specified. In this case, Secrets must be empty.
+	// https://docs.github.com/en/actions/using-workflows/workflow-syntax-for-github-actions#onworkflow_callsecretsinherit
+	InheritSecrets bool
 }
 
 // Job is configuration of how to run a job.
