@@ -1077,6 +1077,7 @@ func (p *parser) parseJob(id *String, n *yaml.Node) *Job {
 	//   - jobs.<job_id>.if
 	//   - jobs.<job_id>.permissions
 
+	// https://docs.github.com/en/actions/using-workflows/reusing-workflows#supported-keywords-for-jobs-that-call-a-reusable-workflow
 	var stepsOnlyKey *String
 	var callOnlyKey *String
 
@@ -1104,7 +1105,6 @@ func (p *parser) parseJob(id *String, n *yaml.Node) *Job {
 			stepsOnlyKey = k
 		case "concurrency":
 			ret.Concurrency = p.parseConcurrency(k.Pos, v)
-			stepsOnlyKey = k
 		case "outputs":
 			ret.Outputs = p.parseOutputs(v)
 			stepsOnlyKey = k
@@ -1154,12 +1154,22 @@ func (p *parser) parseJob(id *String, n *yaml.Node) *Job {
 			}
 			callOnlyKey = k
 		case "secrets":
-			secrets := p.parseSectionMapping("secrets", v, false)
-			call.Secrets = make(map[string]*WorkflowCallSecret, len(secrets))
-			for _, s := range secrets {
-				call.Secrets[s.key.Value] = &WorkflowCallSecret{
-					Name:  s.key,
-					Value: p.parseString(s.val, true),
+			if kv.val.Kind == yaml.ScalarNode {
+				// `secrets: inherit` special case
+				// https://docs.github.com/en/actions/using-workflows/workflow-syntax-for-github-actions#onworkflow_callsecretsinherit
+				if kv.val.Value == "inherit" {
+					call.InheritSecrets = true
+				} else {
+					p.errorf(kv.val, "expected mapping node for secrets or \"inherit\" string node but found %q node", kv.val.Value)
+				}
+			} else {
+				secrets := p.parseSectionMapping("secrets", v, false)
+				call.Secrets = make(map[string]*WorkflowCallSecret, len(secrets))
+				for _, s := range secrets {
+					call.Secrets[s.key.Value] = &WorkflowCallSecret{
+						Name:  s.key,
+						Value: p.parseString(s.val, true),
+					}
 				}
 			}
 			callOnlyKey = k
